@@ -554,6 +554,9 @@ UnixNetVConnection::do_io_close(int alerrno /* = -1 */ )
   write.vio.nbytes = 0;
   write.vio.op = VIO::NONE;
 
+  EThread *t = this_ethread();
+  bool close_inline = !recursion && nh->mutex->thread_holding == t;
+
   INK_WRITE_MEMORY_BARRIER;
   if (alerrno && alerrno != -1)
     this->lerrno = alerrno;
@@ -562,11 +565,8 @@ UnixNetVConnection::do_io_close(int alerrno /* = -1 */ )
   else
     closed = -1;
 
-  if (!recursion) {
-     EThread *t = this_ethread();
-     if (nh->mutex->thread_holding == t)
-       close_UnixNetVConnection(this, t);
-  }
+  if (close_inline)
+    close_UnixNetVConnection(this, t);
 }
 
 void
@@ -706,6 +706,8 @@ UnixNetVConnection::reenable(VIO *vio)
           nh->write_enable_list.push(this);
         }
       }
+	  if (nh->trigger_event && nh->trigger_event->ethread->signal_hook)
+		nh->trigger_event->ethread->signal_hook(nh->trigger_event->ethread);
     } else {
       if (vio == &read.vio) {
         ep.modify(EVENTIO_READ);
@@ -964,7 +966,9 @@ UnixNetVConnection::mainEvent(int event, Event *e)
   MUTEX_TRY_LOCK(rlock, read.vio.mutex ? (ProxyMutex *) read.vio.mutex : (ProxyMutex *) e->ethread->mutex, e->ethread);
   MUTEX_TRY_LOCK(wlock, write.vio.mutex ? (ProxyMutex *) write.vio.mutex :
                  (ProxyMutex *) e->ethread->mutex, e->ethread);
-  if (!hlock || !rlock || !wlock) {
+  if (!hlock || !rlock || !wlock ||
+      (read.vio.mutex.m_ptr && rlock.m.m_ptr != read.vio.mutex.m_ptr) ||
+      (write.vio.mutex.m_ptr && wlock.m.m_ptr != write.vio.mutex.m_ptr)) {
 #ifndef INACTIVITY_TIMEOUT
     if (e == active_timeout)
 #endif
